from enum import Enum
from pathlib import Path

from pydantic import BaseModel, HttpUrl
from pydantic.dataclasses import dataclass


class ContainerizationTypes(Enum):
    NONE = 0
    SINGLE = 1
    MULTIPLE = 2


class ContainerizationEngine(Enum):
    NONE = 0
    DOCKER = 1
    APPTAINER = 2
    BOTH = 3


class ContainerizationFileRepr(BaseModel):
    representation: str


class DependencyTypes(Enum):
    PYPI = "pypi"
    CONDA = "conda"

    @staticmethod
    def get_pypi_url(package_name: str) -> HttpUrl:
        return HttpUrl(f"https://pypi.org/project/{package_name}/")


@dataclass(frozen=True)
class ExperimentDependency:
    dependency_name: str
    url_reference: HttpUrl
    dependency_type: DependencyTypes
    version: str = ""

    def get_name(self) -> str:
        return self.dependency_name

    def any_version_allowed(self) -> bool:
        return self.version == ""


@dataclass(frozen=True)
class ExperimentPrimaryDependencies:
    pypi_dependencies: list[ExperimentDependency]
    conda_dependencies: list[ExperimentDependency]

    def get_pypi_dependencies(self) -> list[ExperimentDependency]:
        return self.pypi_dependencies

    def get_conda_dependencies(self) -> list[ExperimentDependency]:
        return self.conda_dependencies


@dataclass(frozen=True)
class ContainerizationProgramArguments:
    """
    Create a container acting as an isolated environment for execution.
    """

    input_file_path: str
    containerization_type: ContainerizationTypes
    containerization_engine: ContainerizationEngine
    working_directory: Path


@dataclass
class ExecutionProgramArguments:
    """
    Provide information required to execute a process bi-graph.
    """

    input_file_path: str
    interval: float
    output_directory: Path



import logging

from bigraph_schema import Core
from pbsim_common import standard_types
from process_bigraph import allocate_core

logger = logging.getLogger(__name__)

loaded_core: Core | None = None


def set_logging_config(level: str) -> None:
    logging.basicConfig(level=level)


def get_loaded_core() -> Core:
    global loaded_core
    if loaded_core is None:
        loaded_core = allocate_core()
        for k, i in standard_types.items():
            loaded_core.register_type(k, i)
    logger.debug(f"Link registry in use: {loaded_core.link_registry}")
    return loaded_core



from pathlib import Path
from typing import Any

import pytest
from bigraph_schema.core import Core
from process_bigraph.emitter import emitter_from_wires

# from biocompose import standard_types
from pbest.globals import get_loaded_core
from pbest.utils.builder import CompositeBuilder


@pytest.fixture(scope="function")
def fully_registered_core() -> Core:
    return get_loaded_core()


@pytest.fixture(scope="function")
def fully_registered_builder(fully_registered_core) -> CompositeBuilder:
    return CompositeBuilder(core=fully_registered_core)


def _get_model_path() -> str:
    path = Path(__file__).parent.parent
    return f"{path}/resources/BIOMD0000000012_url.xml"


def get_default_config() -> dict[str, Any]:
    return {
        "name": "actin_membrane",
        "random_seed": 0,
    }


@pytest.fixture(scope="function")
def readdy_document() -> dict[str, Any]:
    emitters_from_wires = emitter_from_wires(
        {"particles": ["particles"], "topologies": ["topologies"], "global_time": ["global_time"]},
        address="local:pb_multiscale_actin.processes.simularium_emitter.SimulariumEmitter",
    )

    readd_pbg = {
        "emitter": emitters_from_wires,
        "readdy": {
            "_type": "process",
            "address": "local:pb_multiscale_actin.processes.readdy_actin_membrane.ReaddyActinMembrane",
            "config": get_default_config(),
            "inputs": {"particles": ["particles"], "topologies": ["topologies"]},
            "outputs": {"particles": ["particles"], "topologies": ["topologies"]},
        },
    }
    return {"state": readd_pbg}


@pytest.fixture(scope="function", autouse=True)
def comparison_document() -> dict[Any, Any]:
    model_path = _get_model_path()

    state = {
        # provide initial values to overwrite those in the configured model
        "species_concentrations": {},
        "species_counts": {},
        "comparison_result": {"species_mse": {}},
        "tellurium_step": {
            "_type": "step",
            "address": "local:pbsim_common.simulators.tellurium_process.TelluriumUTCStep",
            "config": {
                "model_source": model_path,
                "time": 10,
                "n_points": 10,
            },
            "inputs": {"concentrations": ["species_concentrations"], "counts": ["species_counts"]},
            "outputs": {
                "result": ["results", "tellurium"],
            },
        },
        "copasi_step": {
            "_type": "step",
            "address": "local:pbsim_common.simulators.copasi_process.CopasiUTCStep",
            "config": {
                "model_source": model_path,
                "time": 10,
                "n_points": 10,
            },
            "inputs": {"concentrations": ["species_concentrations"], "counts": ["species_counts"]},
            "outputs": {
                "result": ["results", "copasi"],
            },
        },
        "comparison": {
            "_type": "step",
            "address": "local:pbsim_common.comparison.MSEComparison",
            "config": {},
            "inputs": {
                "results": ["results"],
            },
            "outputs": {
                "comparison_result": ["comparison_result"],
            },
        },
    }

    document = {"state": state}
    return document
